@@ -723,6 +723,17 @@ plot.trellis <-
     ## packet.number, which is an index to which packet combination is
     ## being used.
 
+
+    ## dev.hold/dev.flush.  There's no clean way to deal with
+    ## the possibility of erroring out before dev.flush() is
+    ## called, because using on.exit() will be complicated
+    ## unless we separate out each page into a separate
+    ## function call.  Instead, we keep track using a flag.
+
+    dev.held <- FALSE # is dev.hold() turned on?
+    on.exit(if(dev.held) dev.flush(), add = TRUE)
+
+
     ## FIXME: what happens when number of pages is 0?
     ## message("no of pages: ", number.of.pages)
 
@@ -731,6 +742,9 @@ plot.trellis <-
         if (TRUE) ## FIXME: remove this after a few versions and reformat
         {
 
+            dev.hold()
+            dev.held <- TRUE
+
             ## In older versions, we used to decide here whether a new
             ## page was actually needed, i.e., whether there is
             ## anything to draw (o.w., why waste a page?).  This is no
@@ -899,7 +913,9 @@ plot.trellis <-
                                                     logsc = x$x.scales$log,
                                                     abbreviate = x$x.scales$abbreviate,
                                                     minlength = x$x.scales$minlength,
-                                                    n = x$x.scales$tick.number,
+                                                    n = x$x.scales$tick.number, # for pretty()
+                                                    nint = x$x.scales$tick.number, # for axisTicks() (log)
+                                                    equispaced.log = x$x.scales$equispaced.log,
                                                     format.posixt = x$x.scales$format)
                             else 
                                 x$xscale.components(lim = x$x.limits[[packet.number]], 
@@ -924,6 +940,8 @@ plot.trellis <-
                                                     abbreviate = x$x.scales$abbreviate,
                                                     minlength = x$x.scales$minlength,
                                                     n = x$x.scales$tick.number,
+                                                    nint = x$x.scales$tick.number,
+                                                    equispaced.log = x$x.scales$equispaced.log,
                                                     format.posixt = x$x.scales$format)
 
 
@@ -945,6 +963,8 @@ plot.trellis <-
                                                     abbreviate = x$y.scales$abbreviate,
                                                     minlength = x$y.scales$minlength,
                                                     n = x$y.scales$tick.number,
+                                                    nint = x$y.scales$tick.number,
+                                                    equispaced.log = x$y.scales$equispaced.log,
                                                     format.posixt = x$y.scales$format)
                             else 
                                 x$yscale.components(lim = x$y.limits[[packet.number]], 
@@ -969,6 +989,8 @@ plot.trellis <-
                                                     abbreviate = x$y.scales$abbreviate,
                                                     minlength = x$y.scales$minlength,
                                                     n = x$y.scales$tick.number,
+                                                    nint = x$y.scales$tick.number,
+                                                    equispaced.log = x$y.scales$equispaced.log,
                                                     format.posixt = x$y.scales$format)
 
 
@@ -1243,8 +1265,17 @@ plot.trellis <-
                                 ## in the original order, not the
                                 ## permuted order
 
-                                strip(which.given = x$perm.cond[i],
-                                      which.panel = which.packet,
+                                which.given <- x$perm.cond[i]
+                                which.panel <- which.packet
+                                
+                                lattice.setStatus(current.which.given =
+                                                  which.given,
+                                                  current.which.panel =
+                                                  which.panel,
+                                                  prefix = prefix)
+                                
+                                strip(which.given = which.given,
+                                      which.panel = which.panel,
 ##                                       panel.number = panel.number,
 ##                                       packet.number = packet.number,
 
@@ -1289,8 +1320,17 @@ plot.trellis <-
                                 ## in the original packet order, not
                                 ## the permuted order
 
-                                strip.left(which.given = x$perm.cond[i],
-                                           which.panel = which.packet,
+                                which.given <- x$perm.cond[i]
+                                which.panel <- which.packet
+                                
+                                lattice.setStatus(current.which.given =
+                                                  which.given,
+                                                  current.which.panel =
+                                                  which.panel,
+                                                  prefix = prefix)
+                                
+                                strip.left(which.given = which.given,
+                                           which.panel = which.panel,
 ##                                            panel.number = panel.number,
 ##                                            packet.number = packet.number,
 
@@ -1311,6 +1351,7 @@ plot.trellis <-
                 }
 
 
+
             ## legend / key plotting
 
             if (!is.null(legend))
@@ -1356,11 +1397,11 @@ plot.trellis <-
                                       full = 
                                       pushViewport(viewport(layout.pos.row = c(1, n.row),
                                                             layout.pos.col = c(1, n.col),
-                                                            name = trellis.vpname("legend", side = "inside", prefix = prefix))),
+                                                            name = trellis.vpname("legend.region", prefix = prefix))),
                                       panel =
                                       pushViewport(viewport(layout.pos.row = range(pos.heights$panel, pos.heights$strip),
                                                             layout.pos.col = range(pos.widths$panel, pos.widths$strip.left),
-                                                            name = trellis.vpname("legend", side = "inside", prefix = prefix))))
+                                                            name = trellis.vpname("legend.region", prefix = prefix))))
                                key.corner <-
                                    if (is.null(legend[[i]]$corner)) c(0,1)
                                    else legend[[i]]$corner
@@ -1372,33 +1413,41 @@ plot.trellis <-
                                    else legend[[i]]$y
                                drawInViewport(key.gf,
                                               viewport(x = unit(key.x, "npc") + unit(0.5 - key.corner[1], "grobwidth", list(key.gf)),
-                                                       y = unit(key.y, "npc") + unit(0.5 - key.corner[2], "grobheight", list(key.gf))))
+                                                       y = unit(key.y, "npc") + unit(0.5 - key.corner[2], "grobheight", list(key.gf)),
+                                                       name = trellis.vpname("legend", side = "inside", prefix = prefix)))
                                upViewport(1)
                            })
-                       }
                 }
-                pushViewport(viewport(layout.pos.row = c(1, n.row),
-                                      layout.pos.col = c(1, n.col),
-                                      name = trellis.vpname("page", prefix = prefix)))
-                if (!is.null(x$page)) x$page(page.number)                
-                upViewport()
-                upViewport()
-            }
-        }
-        if (!is.null(position))
-        {
-            if (!is.null(split))
-            {
-                upViewport()
-                upViewport()
             }
+            pushViewport(viewport(layout.pos.row = c(1, n.row),
+                                  layout.pos.col = c(1, n.col),
+                                  name = trellis.vpname("page", prefix = prefix)))
+            if (!is.null(x$page)) x$page(page.number)                
             upViewport()
+            upViewport()
+
+
+            ## dev.hold/dev.flush.
+            dev.flush()
+            dev.held <- FALSE
         }
-        else if (!is.null(split))
+    }
+    
+    
+    if (!is.null(position))
+    {
+        if (!is.null(split))
         {
             upViewport()
             upViewport()
         }
+        upViewport()
+    }
+    else if (!is.null(split))
+    {
+        upViewport()
+        upViewport()
+    }
 
     lattice.setStatus(current.focus.row = 0,
                       current.focus.column = 0,
@@ -1409,3 +1458,4 @@ plot.trellis <-
 
 
 
+
